'''
Module that contains some functions related to the
vertical grid
'''

#from __future__ import print_function
import numpy as np
import pandas as pd
import xarray as xr
try:
    import pylab as plt
    import matplotlib.patches as patches
except ImportError:
    pass


def extract_weight_grid(config):

    '''
    Extracts weight grid attributes.
    Returns the weight and length step used for integration
    '''

    omega_var_min = float(config["biology.weight.omega.min"])
    omega_var_step = float(config['biology.weight.omega.incr'])
    n_weight_class = int(config['biology.weight.nclass'])
    lmin = float(config['biology.length.min'])
    lmax = float(config['biology.length.max'])
    alpha = float(config['biology.weight.alpha'])
    allom_w_l = float(config['biology.allometric.coeff'])
    c_fonct_w_dep = float(config['biology.functional_response.half_saturation.dependence'])

    omega_var_max = omega_var_min + (n_weight_class - 1) * omega_var_step
    beta = (lmin - lmax * np.power(alpha, (omega_var_min - omega_var_max - 1))) / (1. - np.power(alpha, (omega_var_min - omega_var_max - 1)))
    gam = np.log((lmin - lmax) / (np.power(alpha, omega_var_min) - np.power(alpha, omega_var_max + 1))) / np.log(alpha)

    tmp_length = np.zeros(n_weight_class + 1)
    tmp_weight = np.zeros(n_weight_class + 1)
    weight = np.zeros(n_weight_class)
    length = np.zeros(n_weight_class)
    weight_step = np.zeros(n_weight_class)
    length_step = np.zeros(n_weight_class)
    weight_step2 = np.zeros(n_weight_class + 1)
    w_pow_c = np.zeros(n_weight_class)
    w_pow_m1_3 = np.zeros(n_weight_class)

    for w in range(0, n_weight_class + 1):
        omega_var = omega_var_min + w * omega_var_step
        tmp_length[w] = np.power(alpha, (omega_var + gam)) + beta
        tmp_weight[w] = allom_w_l * np.power(tmp_length[w], 3.)

    for w in range(0, n_weight_class):
        weight[w] = (tmp_weight[w] + tmp_weight[w + 1]) / 2.
        length[w] = np.power((weight[w] / allom_w_l), (1. / 3.))
        w_pow_c[w] = np.power(weight[w], c_fonct_w_dep)
        w_pow_m1_3[w] = np.power(weight[w], (-1. / 3.))


    # computes weigh intervals
    for w in range(0, n_weight_class):
        length_step[w] = tmp_length[w + 1] - tmp_length[w]
        weight_step[w] = tmp_weight[w + 1] - tmp_weight[w]

    weight_step2[0] = weight[0] - tmp_weight[0]
    for w in range(1, n_weight_class):
        weight_step2[w] = weight[w] - weight[w - 1]
    weight_step2[n_weight_class] = tmp_weight[n_weight_class] - weight[n_weight_class - 1]

    return weight_step, length_step


def read_ape_grid(filename):

    '''
    Reads the Apecosm text file containing depth coordinates of
    Apecosm T-grid points.

    :param str filename: Apecosm vertical grid file

    :return: A tuple contaning the depth and thicknessess
     of the cell points

     .. todo::

        Add correction: in the new version, the :samp:`.txt` file 
        provides the lower edge of the T grid

    '''

    data = pd.read_csv(filename, header=None)
    adepth = np.squeeze(data.values)
    nlevel = len(adepth)
    deltaz = np.zeros(nlevel)
    output = np.zeros(nlevel)

    adepth = np.concatenate(([0], adepth))

    for i in range(0, nlevel):
        deltaz[i] = adepth[i + 1] - adepth[i]
        output[i] = 0.5 * (adepth[i + 1] + adepth[i])

    return (output, deltaz)


def partial_step_ape(ape_grid, mesh_file_nemo):

    # Reads the NEMO mesh file to extract all
    # the data that are needed.
    data = xr.open_dataset(mesh_file_nemo)
    tmask = np.squeeze(data['tmask'].values)
    tmask = tmask[0]
    
    adepth, deltaz = read_ape_grid(ape_grid)
    nlevel = len(adepth)
    nlat, nlon = tmask.shape

    output_depth = np.zeros((nlevel, nlat, nlon), dtype=np.float) - 1000
    output_deltaz = np.zeros((nlevel, nlat, nlon), dtype=np.float) - 1000
    output_bottom = np.zeros((nlat, nlon), dtype=np.int)
    output_mask = np.zeros((nlevel, nlat, nlon), dtype=np.int)

    iok = np.nonzero(tmask == 1)

    for ilat, ilon in zip(iok[0], iok[1]):
        temp = _partial_step_ape_grid(adepth, deltaz, data, ilat, ilon)
        output_depth[:, ilat, ilon] = temp['apecosm']['depth']
        output_deltaz[:, ilat, ilon] = temp['apecosm']['deltaz']
        output_bottom[ilat, ilon] = temp['apecosm']['bottom']
        output_mask[:temp['apecosm']['bottom'], ilat, ilon] = 1

    output = {'bottom': output_bottom, 
              'depth': output_depth, 
              'deltaz': output_deltaz,
              'mask': output_mask}
    
    return output


def _partial_step_ape_grid(adepth, deltaz, data_mesh_file_nemo, ilat, ilon):

    '''
    Extracts the vertical NEMO and Apecosm grids at
    point `ilat`, `ilon`.

    '''

    # Hard copy of the adepth and deltaz arguments
    # prevents overwritting
    adepth = adepth.copy()
    deltaz = deltaz.copy()

    nlevel = len(adepth)
    
    if 'e3t_1d' in data_mesh_file_nemo.data_vars.keys():
        # new nemo formatting
        #print('New nemo formatting for vertical scale factors')
        e3t_1d_varname = 'e3t_1d'
        depth_1d_varname = 'gdept_1d'
        e3t_varname = 'e3t_0'
    else:
        # old nemo formatting
        #print('Old nemo formatting for vertical scale factors')
        e3t_1d_varname = 'e3t_0'
        depth_1d_varname = 'gdept_0'
        e3t_varname = 'e3t'

    e3t_1d = data_mesh_file_nemo[e3t_1d_varname].values[0, :]
    znemo = data_mesh_file_nemo[depth_1d_varname].values[0, :]
    tmask = data_mesh_file_nemo['tmask'].values[0, :, ilat, ilon]

    if tmask[0] == 0:
        message = 'The point is in land. Nothing will be done'
        print(message)
        return None

    # check if NEMO grid is in partial step or not
    # nemo_partial_step = 'e3t_0' in data.data_vars.keys()
    nemo_partial_step = (e3t_varname in data_mesh_file_nemo.data_vars.keys())
    #print('NEMO is in partial step mode')

    if nemo_partial_step:
        corrz = data_mesh_file_nemo['hdept'].values[0, ilat, ilon]
        e3t_f = data_mesh_file_nemo[e3t_varname].values[0, :, ilat, ilon]
        e3t = e3t_f
    else:
        e3t = e3t_1d

    # 1D array of size (nlevel+1)
    edges_a = np.append(0, np.cumsum(deltaz))

    # 3D array of size (nlevel_opa+1, NLAT, NLON)
    edges_n = np.append(0, np.cumsum(e3t, axis=0))

    # bottom_opa is the first land point in the vertical
    iok = np.nonzero(tmask == 1)[0]
    bottom_opa = iok[-1] + 1

    nlevel_opa = len(znemo)

    for k in range(0, nlevel_opa):
        if tmask[k] == 0:
            bottom_opa = k
            # if partial step in NEMO, correct the depth of the last ocean point
            if nemo_partial_step:
                znemo[k - 1] = corrz
            break

    # recovers the limit of the land point (i.e. the depth
    # of the NEMO sea/rock interface.
    limit = edges_n[bottom_opa]

    bottom_nico = nlevel
    for k in range(0, nlevel):
        if edges_a[k] > limit:
            bottom_nico = k
            break

    if edges_a[bottom_nico] > limit:
        k = bottom_nico
        edges_a[k] = limit    # // move the edge of the last ocean cell upward

    for k in range(1, nlevel + 1):
        deltaz[k - 1] = edges_a[k] - edges_a[k - 1]   # update in the deltaz value of the last ocean point
        adepth[k - 1] = 0.5 * (edges_a[k] + edges_a[k - 1])  # update in the depth value for the last ocean point

    apecosm = {'deltaz': deltaz, 'depth': adepth, 'edges': edges_a, 'bottom': bottom_nico}
    nemo = {'deltaz': e3t, 'depth': znemo, 'edges': edges_n, 'bottom': bottom_opa, 'limit': limit}
    output = {'nemo': nemo, 'apecosm': apecosm}

    return output


def plot_grid_nemo_ape(ape_grid, mesh_file_nemo, ilat, ilon):

    '''
    Draws the NEMO and Apecosm vertical grid
    at points `ilat`, `ilon`.
    '''
    
    # Reads the NEMO mesh file to extract all
    # the data that are needed.
    data = xr.open_dataset(mesh_file_nemo)
    adepth, deltaz = read_ape_grid(ape_grid)

    output = _partial_step_ape_grid(adepth, deltaz, data, ilat, ilon)
    if output is None:
        return

    nemo = output['nemo']
    apecosm = output['apecosm']
    bottom_nico = apecosm['bottom']
    edges_a = apecosm['edges']
    adepth = apecosm['depth']
    deltaz = apecosm['deltaz']

    bottom_opa = nemo['bottom']
    edges_n = nemo['edges']
    limit = nemo['limit']
    znemo = nemo['depth']
    corrz = znemo[bottom_opa - 1]

    plt.figure(figsize=(4, 10))
    plt.subplots_adjust(right=0.95, left=0.16, bottom=0.1, top=0.95)
    ax1 = plt.gca()

    plt.plot([0]*len(adepth[:bottom_nico]), adepth[:bottom_nico], marker='o', linestyle='none', color='k')
    for i in range(0, len(adepth[:bottom_nico])):
        plt.plot(np.array([-0.5, 0.5, 0.5, -0.5, -0.5]),
                 [edges_a[i], edges_a[i], edges_a[i + 1], edges_a[i + 1], edges_a[i]],
                 color='k')

    plt.plot([0]*len(adepth[bottom_nico:]), adepth[bottom_nico:], marker='o', linestyle='none')
    for i in range(bottom_nico, len(adepth)):
        plt.plot(np.array([-0.5, 0.5, 0.5, -0.5, -0.5]),
                 [edges_a[i], edges_a[i], edges_a[i + 1], edges_a[i + 1], edges_a[i]],
                 color='k')
        rect = patches.Rectangle((-0.5, edges_a[i]), 1, edges_a[i+1] - edges_a[i], color='lightgray', alpha=0.5)
        ax1.add_patch(rect)

    # plot unmasked NEMO
    plt.plot([2 - 0.5] * len(znemo[:bottom_opa - 1]), znemo[:bottom_opa - 1], marker='o', linestyle='none', color='k')
    for i in range(0, len(znemo[:bottom_opa])):
        plt.plot(np.array([1.5, 2.5, 2.5, 1.5, 1.5]) - 0.5,
                 [edges_n[i], edges_n[i], edges_n[i+1], edges_n[i+1], edges_n[i]], color='k')

    # plot masked NEMO
    plt.plot([2 - 0.5] * len(znemo[bottom_opa:]), znemo[bottom_opa:], marker='o', linestyle='none', color='k')
    for i in range(bottom_opa, len(znemo)):
        plt.plot(np.array([1.5, 2.5, 2.5, 1.5, 1.5]) - 0.5,
                 [edges_n[i], edges_n[i], edges_n[i + 1], edges_n[i + 1], edges_n[i]], color='k')
        rect = patches.Rectangle((1, edges_n[i]), 1, edges_n[i+1] - edges_n[i], color='lightgray', alpha=0.5)
        ax1.add_patch(rect)

    plt.plot([2-0.5], corrz, marker='o', linestyle='none', color='firebrick')
    for i in range(bottom_opa-1, bottom_opa):
        plt.plot(np.array([1.5, 2.5, 2.5, 1.5, 1.5]) - 0.5,
                 [edges_n[i], edges_n[i], edges_n[i + 1], edges_n[i + 1], edges_n[i]], color='r', linestyle='--', linewidth=1)
    for i in range(bottom_opa-1, bottom_opa):
        rect = patches.Rectangle((1, edges_n[i+1]), 1, edges_n[i + 1] - edges_n[i + 1], color='lightgray', alpha=0.5)
        ax1.add_patch(rect)

    plt.axhline(limit, color='gold', linestyle='--', linewidth=0.5)

    ylim = edges_n[bottom_opa + 1]
    plt.ylim(0, ylim)

    plt.title('Partial step')
    plt.ylabel('Depth (m)')
    ax1.set_xticks([0, 1.5])
    ax1.set_xticklabels(['Apecosm', 'NEMO'])
    ax1.set_ylim(ax1.get_ylim()[::-1])


if __name__ == '__main__':
#
     filename = 'APECOSM_depth_grid_37.txt'
#     mesh_file = 'data/mesh_mask.nc'
#     ilat, ilon = 115, 73
#     #ilat, ilon = 131, 19
#     ilat, ilon = 131, 44
#
#     #mesh_file = '/home/nbarrier/Modeles/apecosm/svn-apecosm/trunk/tools/config/gyre/mesh_mask.nc'
#     #ilat, ilon = 12,12
#
     adepth, deltaz = read_ape_grid(filename)
#     output = partial_step_ape_grid(filename, mesh_file, ilat, ilon)
#     plot_grid_nemo_ape(filename, mesh_file, ilat, ilon)
#     plt.show()



import sys
import numpy as np
import xarray as xr
from apecosm.misc import compute_daylength
import sys

def get_tcor(temp_array, ta=5020., tref=298.15):
    
    r''' 
    Computes the Ahrrenius temperature as follows:

    .. math::

        \exp^{\frac{T_a}{T_{ref}} - \frac{T_a}{T}}

    :param numpy.array temp_array: Temperature (in :math:`K`)
    '''

    return np.exp(ta / tref - ta / temp_array);


def compute_o2(oxy_array, tcor=1.0, oxyresp=1e5, oxylim=1e-4):

    r''' 
    Computes the oxygen habitat function:

    .. math:: 
       
        H_{oxy}=\frac{1}{1+\exp{\left[OXYRESP \times (T_{cor} \times OXYLIM - Oxy)\right]}}

    :param numpy.array oxy_array: Oxygen (in :math:`mol.L^{-1}`)
    :param numpy.array tcor: Ahrrenius temperature (obtained from the :py:func:`get_tcor` function)
    '''
    
    output = 1. / (1. + np.exp(oxyresp * (tcor * oxylim - oxy_array)));

    return output


def compute_tpref(tcor, sigm_tcor=0.1):
    
    r''' 
    Computes the temperature preference function:

    .. math:: 

        H_{tpref}=\exp{\left[-0.5\left(\frac{\frac{T_{cor}(z)}{T_{cor}(z=0)}-1}{SIGM_{TCOR}}\right)^2\right]} 

    :param numpy.array tcor: Ahrrenius temperature (obtained from the :py:func:`get_tcor` function)

    .. danger:: 

        The temperature array should be 4D, with z as the second dimension.
    
    '''

    if(tcor.ndim != 4):
        message = 'The number of dims must be 4, with z as the second dimension'
        print(message)
        sys.exit(1)

    tcor0 = tcor[:, 0:1, :, :]

    output = np.exp(-.5 * np.power(((tcor / tcor0 - 1.) / sigm_tcor), 2.));

    return output


def compute_tlim(temp_array, t_sup=280.15, t_inf=268.5, ta_lim=200000):

    r'''
    Computes the temperature limitation function:

    .. math:: 

        H_{tlim} = \frac{1}{1+\exp\left(\frac{Ta_{LIM}}{temper}-\frac{Ta_{LIM}}{T_{inf}}\right)} \times \frac{1}{1+\exp\left(\frac{Ta_{LIM}}{T_{sup}}-\frac{Ta_{LIM}}{temper}\right)}

    '''

    output = (1. / (1. + np.exp(ta_lim / temp_array - ta_lim / t_inf))) / (1. + np.exp(ta_lim / t_sup - ta_lim / temp_array));

    return output


def compute_lightpref(par, daylen, opt_light=1.0e2, sigm_light=1.7e2, same_daynight=False, nfact=1e-6):

    r'''
    Computes the light preference function.

    .. math:: 

        \begin{eqnarray}
            & & light & = & \frac{PAR}{DAYLENGTH}*nfactor+EPS \\ 
            H_{lpref} & = & H_{lpred} & = & \frac{mode}{light} \times \exp\left[\frac{(\log(mode)-mu)^2-(\log(light)-mu)^2}{2\times ssigm} \right] 
        \end{eqnarray}

    :param numpy.array par: Photosynthetically Active Radiation (in :math:`W.m^{-2}`). **Should be 4D (time, z, y, x).**
    :param numpy.array daylen: Fraction of day within 24h (can be obtained from the :py:func:`compute_daylength` function). Its shape should allow broadcasting
     with the :samp:`par` array (for instance, if par=(time, depth, y, x), daylen should be (time, 1, y, x). Should correspond with the time step of the PAR files.
    :param bool same_daynight: Whether light habitat should distinguish day and night cycles.
    :param float nfactor: Proportionality coefficient between light intensity at night and during the day.

    :return: An array of dimensions (dn, time, z, y, x), with dn=0 for daytime, dn=1 for night time.

    .. note:: 

        This function can also be used to compute the light contribution to the functional response.

    '''

    nfactor = np.ones((2), dtype=np.float)   # init the nfactor with ones 
    
    if(not(same_daynight)):
        nfactor[1] = nfact    # if not same_daynight -> change nfactor for night (dn = 1)

    nfactor = nfactor[:, np.newaxis, np.newaxis, np.newaxis, np.newaxis]   # dn, time, z, y, x

    mu = np.log(opt_light) - .5 * np.log1p((sigm_light * sigm_light) / (opt_light * opt_light));
    sigm = np.sqrt(np.log1p(sigm_light * sigm_light / (opt_light * opt_light)));
    ssigm = sigm * sigm;
    mode = np.exp(mu - ssigm);

    par = np.ma.masked_where(np.isnan(par), par)
    par = par[np.newaxis, :, :, :, :]
    daylen = daylen[np.newaxis, :, :, :, :]
    daylen[daylen < 1./24.] = 1 / 24.
    daylen = np.ma.masked_where(par[:, :, 0:1, :, :].mask, daylen)
    tmplight = np.divide(par, daylen, where=(par.mask == False)) * nfactor + 2 * sys.float_info.min 

    output = np.divide(mode, tmplight, where=(tmplight.mask==False)) * np.exp((np.power((np.log(mode) - mu), 2.) - np.power((np.log(tmplight, where=(tmplight.mask==False)) - mu), 2.)) / (2. * ssigm));

    return output



if __name__ == '__main__':

    dirin = '/home/nbarrier/Modeles/apecosm/svn-apecosm/trunk/tools/config/gyre_nico/data'
    filelist = "%s/clim_grid_T.nc" %dirin
    data = xr.open_dataset(filelist)
    temp = data['votemper'].values + 273.15   # converted into K
    lat = data['nav_lat'].values

    # computation
    tcor = get_tcor(temp)

    filelist = "%s/clim_ptrc_T.nc" %dirin
    data = xr.open_dataset(filelist)
    oxy = data['O2'].values * 1e-6   # converted into mmol/L
    par = data['PAR'].values 
    
    ######################## group = 0
    tpref = compute_tpref(tcor)
    tpref = np.ma.masked_where(temp == 273.15, tpref)
    o2hab = compute_o2(oxy, tcor)
    
    ######################## group = 0
    daylen = compute_daylength(lat)   # doy, lat, lon
    time = np.arange(par.shape[0]) * 5 + 2.5
    time = time.round().astype(np.int)
    daylen = daylen[time, np.newaxis, :, :]
    daylen[daylen < 1 / 24.] = 1/24.

    lightpref = compute_lightpref(par, daylen, opt_light=8e-4, sigm_light=8e-3, same_daynight=True)
    lightpref = compute_lightpref(par, daylen, opt_light=8e-4, sigm_light=8e-3, same_daynight=False)

    '''
    import pylab as plt
    temp = np.ma.masked_where(temp == 273.15, temp)
    oxy = np.ma.masked_where(oxy == 0, oxy)
    o2hab = np.ma.masked_where(oxy.mask, o2hab)
    print o2hab.min(), o2hab.max()
    print oxy.min(), oxy.max()

    iz = 0
    it = 50

    plt.figure()
    ax = plt.subplot(211)
    cs = plt.imshow(oxy[it, iz])
    plt.colorbar(cs)
    ax = plt.subplot(212)
    cs = plt.imshow(o2hab[it, iz])
    cs.set_clim(0, 1)
    plt.colorbar(cs)
    plt.savefig('toto.png')
    '''





'''
Module dedicated to the computation and
plotting of size spectra
'''


from __future__ import print_function
import sys
import numpy as np
try:
    import pylab as plt
except ImportError:
    pass
import xarray as xr
import apecosm.constants as co
import apecosm.misc
import apecosm.conf


def compute_spectra_ltl(data, L, N=100, conv=1e-3, output_var='weight'):

    r'''
    Computes the size/weight spectra for lower trophic levels
    (i.e. PISCES variables).

    **Weight spectra**:

    .. math::

        P(w) = a w^{-1}

    .. math::

        PHY2 = a\int_{w_1}^{w_2} w^{-1} dw = a \ln\left(\frac{w_2}{w_1}\right)

    .. math::

        a = \frac{PHY2}{\ln\left(\frac{w_2}{w_1}\right)}

    **Length spectra**:

    .. math::

        P(l) = b l^{-1}

    .. math::

        PHY2 = b\int_{l_1}^{l_2} l^{-1} dl = a \ln\left(\frac{l_2}{l_1}\right)

    .. math::

        b = \frac{PHY2}{\ln\left(\frac{l_2}{l_1}\right)}


    :param numpy.array|float data: LTL content (number of moles), either a float or a 1D array
    :param list L: Size limits of the LTL class
    :param int N: Number of weight class to draw.
    :param float conv: Conversion factor to move from LTL units to mol/m3 (Apecosm units)
    :param str out: 'weight' if weight spectra should be drawn, else 'length' for a
     size spectra

    :return: A tuple containing the weight, length array and the corresponding biomass

    '''

    # multiply conversion factor from C to E
    conv *= co.C_E_CONVERT
    data *= conv

    if isinstance(data, xr.DataArray):
        print('Data is converted into a numpy array')
        data = data.values
        data = np.ma.masked_where(np.isnan(data), data)
        data = np.atleast_1d(data)

    # L is a 2d array
    L = np.sort(L)
    if (L.ndim != 1) | (len(L) != 2):
        message = 'The "L" argument must be a list or array with two values'
        print(message)
        sys.exit(0)

    # if float provided, converted into array for calculation convenience.
    if isinstance(data, np.float64):
        data = np.atleast_1d(data)

    if data.ndim != 1:
        message = 'Number of dimensions must be one.'
        print(message)
        sys.exit(0)

    # Convert L into W using allometric formulae
    W = apecosm.misc.size_to_weight(L)

    # generate a vector of weights and length
    wvec = np.linspace(W[0], W[1], N)
    lvec = np.linspace(L[0], L[1], N)

    # Convert W, L and data into consistent arrays
    data2d, wvec2d = np.meshgrid(data, wvec)
    data2d, lvec2d = np.meshgrid(data, lvec)

    alpha = data2d / np.log(W[1] / W[0])
    beta = data2d / np.log(L[1] / L[0])
    rho = alpha * np.power(wvec2d, -1)
    rhoL = beta * np.power(lvec2d, -1)

    if output_var == 'weight':
        return wvec2d, rho
    else:
        return lvec2d, rhoL


def plot_oope_spectra(data, output_var='length', config=None, **kwargs):

    r'''
    Plots the OOPE size spectra. Since OOPE data are
    stored in :math:`J.m^{-3}.kg^{-1}`, they must me converted
    into :math:`J.m^{-3}.m^{-1}`:

    .. math::

        a \int_{w_i}^{w_{i+1}} w^{-1} dw = b \int_{l_i}^{l_{i+1}} l^{-3} dl

    .. math::

        \xi(w_i) \delta w_i = \xi(l_i) \delta l_i

    .. math::

        \xi(l_i) = \xi(w_i) \frac{\delta w_i}{\delta l_i}

    :param xarray.Dataset data: A OOPE dataset that must be
     2-dimensional (community, weight)
    :param str output_var: 'length' if size spectra should
     be plotted, else weight spectra is plotted.

    return None

    '''

    message = 'The input dataset must be have two dimensions '
    message += '(community, weight)'

    if data['OOPE'].ndim != 2:
        print(message)
        sys.exit(0)

    if 'community' not in data['OOPE'].dims:
        print(message)
        sys.exit(0)

    if 'weight' not in data['OOPE'].dims:
        print(message)
        sys.exit(0)

    # recovers the default cmap
    cmap = getattr(plt.cm, plt.rcParams['image.cmap'])

    oope = data['OOPE'].values
    weight = data['weight'].values
    length = data['length'].values
    commnames = apecosm.misc.extract_community_names(data)
    comm = data['community'].values.astype(np.int)

    if output_var == 'length':
        wstep, lstep = apecosm.grid.extract_weight_grid(config)
        oope = oope * wstep[np.newaxis, :] / lstep[np.newaxis, :]
        xvar = length
    else:
        xvar = weight

    ax = plt.gca()
    for icom in comm:
        color = cmap(float(icom) / len(comm))
        ax.scatter(xvar, oope[icom], c=color, label=commnames[icom], **kwargs)


def set_plot_lim():

    '''
    Automatically sets the plot limits.
    It loops over all the scatter plots and
    line plots and extracts the bounding plot

    '''

    ymin = np.Inf
    xmin = np.Inf

    ymax = np.NINF
    xmax = np.NINF

    # Loop over all the collections, i.e. over all the scatter
    # plots
    for coll in plt.gca().collections:

        path = coll.get_offsets()

        xmin = np.min([path[:, 0].min(), xmin])
        ymin = np.min([path[:, 1].min(), ymin])
        xmax = np.max([path[:, 0].max(), xmax])
        ymax = np.max([path[:, 1].max(), ymax])

    # Loop over all the lines
    for l in plt.gca().get_lines():

        path = np.array([l.get_xdata(), l.get_ydata()]).T

        xmin = np.min([path[:, 0].min(), xmin])
        ymin = np.min([path[:, 1].min(), ymin])
        xmax = np.max([path[:, 0].max(), xmax])
        ymax = np.max([path[:, 1].max(), ymax])

    plt.gca().set_ylim(ymin, ymax)
    plt.gca().set_xlim(xmin, xmax)

# if __name__ == '__main__':
#
#     from extract import extract_time_means, extract_ltl_data
#     import xarray as xr
#
#     config = apecosm.conf.read_config('/home/nbarrier/Modeles/apecosm/svn-apecosm/trunk/tools/config/gyre/oope.conf')
#     dirin = '../../../doc_user/data/'
#
#     meshfile = '%s/mesh_mask.nc' % dirin
#     domain = 'benguela'
#
#     test = xr.open_dataset('test_ts.nc')
#     test = test.mean(dim='time')
#
#     file_pattern = '%s/PISCES*nc' % dirin
#     dataPHY2 = extract_ltl_data(file_pattern, 'PHY2', meshfile, domain)
#     dataZOO = extract_ltl_data(file_pattern, 'ZOO', meshfile, domain)
#     dataZOO2 = extract_ltl_data(file_pattern, 'ZOO2', meshfile, domain)
#     dataGOC = extract_ltl_data(file_pattern, 'GOC', meshfile, domain)
#
#     [dataPHY2, dataZOO, dataZOO2, dataGOC] = map(extract_time_means, [dataPHY2, dataZOO, dataZOO2, dataGOC])
#
#     output_var = 'weight'
#
#     L = [10e-6, 100e-6]
#     lVec2dPHY2, rhoLPHY2 = compute_spectra_ltl(dataPHY2['PHY2'], L, output_var=output_var)
#
#     L = [20.e-6, 200.e-6]
#     lVec2dZOO, rhoLZOO = compute_spectra_ltl(dataZOO['ZOO'], L, output_var=output_var)
#
#     L = [200.e-6, 2000.e-6]
#     lVec2dZOO2, rhoLZOO2 = compute_spectra_ltl(dataZOO2['ZOO2'], L, output_var=output_var)
#
#     L = [100e-6, 50000.e-6]
#     lVec2dGOC, rhoLGOC = compute_spectra_ltl(dataGOC['GOC'], L, output_var=output_var)
#
#     plt.figure()
#     ax = plt.gca()
#     plot_oope_spectra(test, output_var=output_var, config=config)
#
#     ax.scatter(lVec2dPHY2, rhoLPHY2, label='PHY2')
#     ax.scatter(lVec2dZOO, rhoLZOO, label='ZOO')
#     ax.scatter(lVec2dZOO2, rhoLZOO2, label='ZOO2')
#     ax.scatter(lVec2dGOC, rhoLGOC, label='GOC')
#     set_plot_lim()
#     ax.set_yscale("log")
#     ax.set_xscale("log")
#     plt.legend()
#
#     plt.savefig('size_spectra_%s_%s.png' % (domain, output_var), bbox_inches='tight')



''' Module that contains some miscellaneous functions '''

import re
import numpy as np
import apecosm.constants as constants

def find_percentile(data, percentage=1):

    ''' 
    Extract percentile to saturate the colormaps.
    They are computed from unmasked arrays

    :param numpy.array data: Data array
    :param float percentage: Percentage used to
     saturate the colormap.

    :return: A tuple containing the lower and upper bounds (cmin, cmax)

    '''
    data = np.ma.masked_where(np.isnan(data), data)
    iok = np.nonzero(np.logical_not(np.ma.getmaskarray(data)))
    temp = data[iok]

    cmin = np.percentile(np.ravel(temp), percentage)
    cmax = np.percentile(np.ravel(temp), 100 - percentage)

    return cmin, cmax


def compute_daylength(lat, nlon=None):

    '''
    Computes the day-length fraction providing a latitude array by
    using the same formulation as in APECOSM.

    :param numpy.array lat: Latitude array (either 1D or 2D)
    :param int nlon: Number of longitudes
    :return: A 2D array with the daylength fraction
    '''

    lat = np.squeeze(lat)

    # If the number of dimensions for lat is 1,
    # tile it with dimensions (nlat, nlon)
    if lat.ndim == 1:
        lat = np.tile(lat, (nlon, 1)).T

    time = np.arange(0, 365)

    nlon = lat.shape[1]

    lat = lat[np.newaxis, :, :]
    time = time[:, np.newaxis, np.newaxis]

    p = 0.833

    theta = 0.2163108 + 2 * np.arctan(0.9671396 * np.tan(0.00860 * (time + 1 - 186)))  # eq. 1
    phi = np.arcsin(0.39795 * np.cos(theta))                                       # eq. 2
    a = (np.sin(p * np.pi / 180.) + np.sin(lat * np.pi / 180.) * np.sin(phi)) / (np.cos(lat * np.pi / 180.) * np.cos(phi))
    a[a >= 1] = 1
    a[a <= -1] = -1
    daylength = 1.0 - (1.0 / np.pi) * np.arccos(a)
    daylength[daylength < 0] = 0
    daylength[daylength > 1] = 1

    return daylength


def extract_community_names(data):

    '''
    Extracts community names from the units attribute in
    the OOPE NetCDF file. It uses regular expressions to extract
    the names. `community` must be a variable in the NetCDF
    variable.

    :param xarray.Dataset data: xarray dataset that
     is returned when using xr.open_dataset on the
     output file.

    :return: The list of community names
    '''

    # extract community and reconstruct community name
    # from community units
    comm = data['community']
    units = comm.units
    comm = comm.values.astype(np.int)
    comm_string = []
    for p in comm:
        pattern = '.*%d=([a-z]+).*' % p
        regexp = re.compile(pattern)
        test = regexp.match(units)
        if test:
            comm_string.append(str(test.groups()[0]))

    return comm_string


def size_to_weight(size):

    r'''
    Converts size (in m) into weight (in kg) #

    .. math::

        W = A L^3

    '''

    return constants.ALLOM_W_L * np.power(size, 3)


def weight_to_size(weight):

    r'''
    Converts weight (in kg) into size (in m)

    ..math::

        L = \left(\frac{W}{A}\right)^{-3}

    '''

    return np.power(weight / constants.ALLOM_W_L, 1/3.)


if __name__ == '__main__':

    X = 1e-6
    W = size_to_weight(X)
    XTEST = weight_to_size(W)



''' Module that handles the processing of configuration files '''

from __future__ import print_function
import re
import os.path
import numpy as np

SEP_LIST = [';', '=', ',', '\t', ' ']


def read_config(filename, relative_to_main=False, params=None, path=''):

    '''
    Reads configuration files.

    :param str filename: The main configuration file.
    :param boolean relative_to_main: True if path names are related to
     the main configuration file, False if path names are related to
     the current configuration file.
    :param dict params: Dictionnary that contains all the parameters.
    :param str path: Path that is used to reconstruct the
     sub-configuration file names.

    :return: A dictionnary containing all the parameters.

    '''

    if params is None:
        params = {}

    # reconstructs the config file name from path argument and filename
    filename = os.path.join(path, filename)

    # if config file is main, initialises the path from main configuration file
    if path == '':
        path = os.path.dirname(filename)

    if not relative_to_main:
        # if path names are related to local path names
        # extracts the new paths
        path = os.path.dirname(filename)

    # Opens the configuration file and extracts the lines
    with open(filename) as file_in:
        lines = file_in.readlines()

    # removes blank spaces at the beginning and end of the lines
    lines = [l.strip() for l in lines]

    # extracts the pattern that matches configuration parameters
    pattern = '^[a-zA-Z]+'
    regexp = re.compile(pattern)

    # loop over the lines
    for l in lines:

        # if param is matched
        if regexp.match(l):

            sep = _find_separator(l)

            if sep is None:
                # if seperator is not found, nothing is done
                message = 'The line %s could not have been processed' % l
                print(message)
                continue

            # extract key and value, remove spurious white spaces
            key, val = l.split(_find_separator(l))
            key = key.strip()
            val = val.strip()

            # if parameter is a new configuration file, read config
            if key.startswith('apecosm.configuration'):
                read_config(val, relative_to_main=relative_to_main, params=params, path=path)
            else:
                # else, if parameter is not already defined, it is
                # added to the parameter list
                if key not in params.keys():
                    params[key] = _convert(val)
                else:
                    message = 'Parameter %s is already defined and equal to %s.\n' % (key, params[key])
                    message += 'Current value %s is ignored.' % val
                    print(message)

    return params


def _convert(val):
    
    # try the conversion into float
    if ',' in val:
        
        val = val.split(',')
        val = [_convert(v) for v in val]
        
        # check if any of the param is a string
        # if so, all vals are converted into str
        test_str = [isinstance(v, str) for v in val]
        if(np.any(test_str)): 
            val = [str(v) for v in val]
            return val

        # if no string, looking for float
        test_float = [isinstance(v, float) for v in val]
        if(np.any(test_float)): 
            val = [float(v) for v in val]
            return val

        return val

    
    # try the conversion into int/bool
    try:
        val = int(val)
        return val
    except ValueError:
        pass
    
    # try the conversion into float
    try:
        val = float(val)
        return val
    except ValueError:
        pass
    
    # try the conversion into float
    try:
        val = float(val)
        return val
    except ValueError:
        pass
    
    return val



def _find_separator(string):

    '''
    Returns the key-value separator for
    a given string. It is defined as the separator
    that splits the string into two parts.

    :param str string: Input string

    :return: The key-value separator. If no separator is found,
    returns None.
    '''

    for sep in SEP_LIST:

        if len(string.split(sep)) == 2:

            return sep

    return None

if __name__ == '__main__':

    dirin = '/home/nbarrier/Modeles/apecosm/svn-apecosm/trunk/tools/config/gyre/'
    dirin = 'config/'
    filename = 'oope.conf'

    x = 'sl, alt, alt'
    x = '2, 1.0'
    toto = _convert(x)
    print(toto)
    print(type(toto[0]))
    print(type(toto[1]))

    #conf = read_config(dirin + filename, relative_to_main=True)
    #print conf



# -*- coding: utf-8 -*-
"""

Apecosm Python License CeCill

"""

from __future__ import print_function

import os

import pkg_resources  # part of setuptools
try:
    __version__ = pkg_resources.require("apecosm")[0].version
except:
    VERSION_FILE = os.path.join('{0}/../'.format(os.path.dirname(__file__)), 'VERSION')
    with open(VERSION_FILE, 'r') as infile: 
        __version__ = infile.read().strip()

__description__ = "Python package for the manipulation of the Apecosm model"
__author_email__ = "nicolas.barrier@ird.fr"

from .conf import read_config
from .domains import inpolygon, plot_domains
from .extract import extract_ltl_data, extract_time_means, extract_oope_data
from .misc import find_percentile, compute_daylength, extract_community_names, size_to_weight, weight_to_size
from .netcdf import rebuild_restart
from .size_spectra import compute_spectra_ltl, plot_oope_spectra, set_plot_lim
from .mplot import plot_oope_map, plot_season_oope 
from .grid import extract_weight_grid, read_ape_grid, plot_grid_nemo_ape, partial_step_ape
from .habitat import get_tcor, compute_o2, compute_tpref, compute_tlim, compute_lightpref



''' Module containing some functions related to NetCDF files'''

from __future__ import print_function
from glob import glob
import sys
import os
import xarray as xr
import numpy as np


def rebuild_restart(dirin):

    '''
    Rebuilds a global restart file from individual restart
    files. Usefull if the tile file has been deleted by mistake.

    :param str dirin: Name of the restart directory.

    '''

    filelist = np.sort(glob("%s/*restart*.nc.*" % dirin))

    if filelist.size == 0:
        print("No restart files was found in %s" % dirin)
        print("The program will be stopped")
        sys.exit(os.EX_NOINPUT)

    file_number = 0

    # Loop over all the tiles
    for filename in filelist[:]:

        if file_number == 0:
            # The first file is used to initialize output array
            print("Initializing the output variable from " + filename)
            data = xr.open_dataset(filename)
            output_name = filename.replace('.nc.000', '.nc')
        else:
            # The next files (1, 2, ..., ntile - 1) are used to concatenate
            # the output array
            print("Concatenating the output variable from " + filename)
            temp = xr.open_dataset(filename)   # open the current tile restart
            data = xr.concat([data, temp], 'cell')   # append it to the data variable

        file_number += 1

    # Save the output into a NetCDF file
    print('Reconstructed restart is in %s' % output_name)
    data.to_netcdf(output_name)



'''
Module that contains functions for plotting
OOPE maps. The Ngl module is needed. It
can be used by using a virtual environment
(see https://www.pyngl.ucar.edu/)
'''

from __future__ import print_function
import sys
import os.path
import numpy as np
import xarray as xr
try:
    import Ngl
except ImportError:
    pass
import apecosm.extract
import apecosm.misc as misc


def plot_oope_map(data, figname, size_class=None, percentage=1):

    ''' Draws 2D OOPE maps.

    :param xarray.Dataset data: 2D OOPE array. Dims must be (y, x, comm, size)
    :param str figname: Name of the figure file (must end by .png or .pdf)
    :param list size_class: Size classes to output (in m)
    :param float percentage: percentage used to saturate colorbar from percentile.
     Colorbar is saturated from values of the (X) and (100 - X) percentile.

    :return: None
    
    '''

    if size_class is None:
        size_class = [1e-3, 1e-2, 1e-1, 1]

    # sort size class in ascending order, and add 0 and infinity as size bounds
    size_class = np.sort(size_class)

    if size_class[0] != 0:
        size_class = np.concatenate(([0], size_class), axis=0)
    if size_class[-1] != np.Inf:
        size_class = np.concatenate((size_class, [np.Inf]), axis=0)

    # Check that the OOPE dataset has 4 dimensions (i.e. no time dimension)
    ndims = len(data['OOPE'].dims)

    if ndims != 4:
        message = 'Data must have dimensions of size (lat, lon, comm, wei)'
        print(message)
        sys.exit(0)

    # Recover data variables
    length = data['length'].values
    oope = data['OOPE'].values
    lon = data['longitude'][:].values
    lat = data['latitude'][:].values
    comm = data['community'][:].values.astype(np.int)

    # mask oope where land
    oope = np.ma.masked_where(np.isnan(oope), oope)

    comm_string = misc.extract_community_names(data)

    if figname.endswith('png'):
        form = 'png'
    elif figname.endswith('pdf'):
        form = 'pdf'
    else:
        message = 'Figure name should end with png or pdf'
        print(message)
        sys.exit(0)

    # opens the document
    wks = Ngl.open_wks(form, figname)

    # set the document colormap
    # resngl = Ngl.Resources()
    # resngl.wkColorMap = 'precip2_15lev'
    # Ngl.set_values(wks, resngl)

    # Add gray to the workspace
    Ngl.new_color(wks, 0.7, 0.7, 0.7)

    # init the plot resources
    # For a detailed description, see https://www.ncl.ucar.edu/Document/Graphics/Resources/
    res = Ngl.Resources()

    # not necessary, just a good habit
    res.nglDraw = False
    res.nglFrame = False

    # Set map resources.
    res.mpLimitMode = "LatLon"     # limit map via lat/lon
    res.mpMinLatF = lat.min()         # map area
    res.mpMaxLatF = lat.max()         # latitudes
    res.mpMinLonF = lon.min()         # and
    res.mpMaxLonF = lon.max()         # longitudes
    res.mpFillOn = True
    res.mpLandFillColor = "LightGray"
    res.mpOceanFillColor = -1
    res.mpInlandWaterFillColor = "LightBlue"
    res.mpGeophysicalLineThicknessF = 1  # thickness of coastlines

    # coordinates for contour plots
    res.sfXArray = lon
    res.sfYArray = lat

    res.cnFillOn = True  # filled contour
    res.cnLinesOn = False  # no lines
    res.cnLineLabelsOn = False  # no labels
    res.cnInfoLabelOn = False  # no info about contours

    res.cnFillMode = 'CellFill'  # contourf=AreaFill, pcolor="CellFill" or "RasterFill"
    res.lbOrientation = "Horizontal"  # colorbar orientation
    res.lbLabelFontHeightF = 0.012  # colorbar label fontsize
    res.lbTitlePosition = "Bottom"  # position of colorbar title
    res.lbTitleFontHeightF = 0.012  # title font height

    # res.cnFillPalette = "wgne15"
    res.cnFillPalette = "WhiteBlueGreenYellowRed"

    txres = Ngl.Resources()
    txres.txJust = "BottomCenter"
    txres.txFontHeightF = 0.02

    res.cnLevelSelectionMode = 'ExplicitLevels'
    res.cnMaxLevelCount = 41
    res.mpGridAndLimbOn = True

    # Equations are complicated with NCARG
    # see https://www.ncl.ucar.edu/Applications/fcodes.shtml
    res.lbTitleString = "OOPE (J.kg~S~-1~N~.m~S~-2~N~)"  # title of colorbar

    # Loop over communities
    for icom in comm:
        # Loop over size classes
        for isize in xrange(0, len(size_class) - 1):

            # Extract sizes comprised between the size class bound
            iw = np.nonzero((length >= size_class[isize]) & (length < size_class[isize+1]))[0]
            if iw.size == 0:
                continue

            # Integrate OOPE for the given community and given size class
            temp = oope[:, :, icom, iw]
            temp = np.sum(temp, axis=-1)

            # Finds the colorbar limits
            cmin, cmax = misc.find_percentile(temp, percentage=1)

            # draw the contour maps
            # defines the contour
            res.cnLevels = np.linspace(cmin, cmax, res.cnMaxLevelCount)
            mapplot = Ngl.contour_map(wks, temp, res)

            # add title
            title = 'Community=%s, %.2E m <= L < %.2E m' % (comm_string[icom], size_class[isize], size_class[isize + 1])
            Ngl.text_ndc(wks, title, 0.5, 0.85, txres)

            # draws the map
            Ngl.draw(mapplot)

            # add a page to the pdf output
            Ngl.frame(wks)

            # WARNING!!! After calls to this function,
            # Ngl.end() function has to be called!


def plot_season_oope(file_pattern, figname, percentage=1):

    ''' Plot seasonal means

    :param str file_pattern: File pattern (for instance, "data/\*nc")
    :param str figname: Figure name
    :param str percentage: Percentile for colormap saturation

    :return: None

    '''

    fig_dir = os.path.dirname(figname)
    fig_name = os.path.basename(figname)

    data = xr.open_mfdataset(file_pattern)

    clim = extract.extract_time_means(data, time='season')
    for s in clim['season'].values:

        print('++++++++++++++++ Drawing season %s ' % s)

        temp = clim.sel(season=s)
        outfile = '%s/%s_%s' % (fig_dir, s, fig_name)
        plot_oope_map(temp, outfile, percentage=percentage)

    Ngl.end()


if __name__ == '__main__':

    plot_season_oope('data/CMIP2_SPIN_OOPE_EMEAN.nc', './OOPE_mean.pdf')



import numpy as np

def get_monthly_clim(var):
    
    ntime = var.shape[0]

    nyears = ntime // 12

    index = np.arange(12)

    for i in range(nyears):
        if(i == 0):
            clim = var[index]
        else:
            clim += var[index]
        index += 12
    clim /= nyears

    anom = np.zeros(var.shape)
    index = np.arange(12)
    for i in range(nyears):
        anom[index] = var[index] - clim
        index += 12

    return clim, anom



''' Module that contains some functions for data extractions '''

from __future__ import print_function
import sys
import xarray as xr
import numpy as np
from .domains import DOMAINS, inpolygon


def extract_ltl_data(file_pattern, varname, meshfile, domain_name, depth_max=None):

    '''
    Extraction of LTL values on a given domain.
    LTL is vertically integrated and spatially integrated over
    the domain.

    :param str file_pattern: LTL file pattern
    :param str varname: LTL variable name
    :param str meshfile: Name of the NetCDF meshfile
    :param str domain_name: Name of the domain to extract

    :return: A xarray dataset

    '''

    # open the dataset
    dataset = xr.open_mfdataset(file_pattern)
    data = dataset[varname].values
    data = np.ma.masked_where(np.isnan(data), data)

    # open the mesh file, extract tmask, lonT and latT
    mesh = xr.open_dataset(meshfile)
    e3t = mesh['e3t'].values
    e2t = mesh['e2t'].values
    e1t = mesh['e1t'].values

    surf = e1t * e2t
    surf = surf[np.newaxis, :, :, :]

    tmask = mesh['tmask'].values
    lon = np.squeeze(mesh['glamt'].values)
    lat = np.squeeze(mesh['gphit'].values)

    # extract the domain coordinates
    if isinstance(domain_name, str):
        domain = DOMAINS[domain_name]
    else:
        domain = domain_name

    # generate the domain mask
    maskdom = inpolygon(lon, lat, domain['lon'], domain['lat'])

    # add virtual dimensions to domain mask and
    # correct landsea mask
    maskdom = maskdom[np.newaxis, np.newaxis, :, :]
    tmask = tmask * maskdom

    temp = surf * e3t * tmask * data

    if depth_max is not None:
        iok = np.nonzero(np.squeeze(mesh['gdept_0'].values) < depth_max)[0]
        temp = temp[:, iok, :, :]

    # integrate spatially and vertically the LTL concentrations
    data = np.sum(temp, axis=(1, 2, 3))
    # output the time series as a Dataset in order to keep track of
    # the coordinates (community and weight)
    output = xr.Dataset({varname: (['time_counter'], data)})
    output['time_counter'] = dataset['time_counter']

    return output


def extract_time_means(data, time=None):

    ''' 
    Extract data time mean.

    :param str file_pattern: File pattern (for instance, "data/\*nc")
    :param str time: Time mean. Can be time average ('tot'), 
     yearly means ('year'), seasonal means ('season') or monthly means
     ('monthly')

    :return: A data array with the given time mean
    '''

    if (time is not None) & (time not in ('season', 'month', 'year')):
        message = "Time argument must be set to 'season', 'month', 'year' or None"
        print(message)
        sys.exit(0)

    if 'time_counter' in data.dims.keys():
        dimname = 'time_counter'
    else:
        dimname = 'time'

    if time is None:
        climatology = data.mean(dimname)
    else:
        climatology = data.groupby('time.%s' % time).mean(dimname)

    return climatology


def extract_oope_data(file_pattern, varname, meshfile, domain_name):

    '''
    Extraction of OOPE values on a given domain.
    OOPE is spatially integrated over
    the domain.

    :param str file_pattern: OOPE file pattern
    :param str varname: OOPE variable name
    :param str meshfile: Name of the NetCDF meshfile
    :param str domain_name: Name of the domain to extract

    :return: A tuple with the time-value and the LTL time series

    '''

    # open the dataset
    dataset = xr.open_mfdataset(file_pattern)
    data = dataset[varname].values
    data = np.ma.masked_where(np.isnan(data), data)

    # open the mesh file, extract tmask, lonT and latT
    mesh = xr.open_dataset(meshfile)
    e2t = mesh['e2t'].values
    e1t = mesh['e1t'].values

    surf = e1t * e2t
    surf = surf[:, :, :, np.newaxis, np.newaxis]

    tmask = mesh['tmask'].values
    lon = np.squeeze(mesh['glamt'].values)
    lat = np.squeeze(mesh['gphit'].values)

    # extract the domain coordinates
    if(isinstance(domain_name, str)):
        domain = DOMAINS[domain_name]
    else:
        domain = domain_name

    # generate the domain mask
    maskdom = inpolygon(lon, lat, domain['lon'], domain['lat'])

    # add virtual dimensions to domain mask and
    # correct landsea mask
    maskdom = maskdom[np.newaxis, np.newaxis, :, :]
    tmask = tmask * maskdom
    tmask = tmask[:, 0, :, :, np.newaxis, np.newaxis]

    # check whether orca grid or not (i.e. the lat dimension has 1 row missing in OOPE)
    if tmask.shape[1] == data.shape[1] + 1:
        message = 'Assuming that working on Orca grid.\n'
        message += 'Last row of mask and surface is removed.'
        print(message)
        surf = surf[:, :-1, :, :, :]
        tmask = tmask[:, :-1, :, :, :]

    # integrate spatially the OOPE concentrations
    data = np.sum(surf * tmask * data, axis=(1, 2))

    # output the time series as a Dataset in order to keep track of
    # the coordinates (community and weight)
    output = xr.Dataset({'OOPE': (['time', 'community', 'weight'], data)})
    output['community'] = dataset['community']
    output['weight'] = dataset['weight']
    output['time'] = dataset['time']
    output['length'] = dataset['length']

    return output


# if __name__ == '__main__':
#
#     meshfile = 'data/mesh_mask.nc'
#     domain = 'benguela'
#
#     #output = extract_oope_data('data/CMIP2_SPIN_OOPE_EMEAN.nc', 'OOPE', meshfile, domain)
#     #output.to_netcdf('test_ts.nc', format='NETCDF4_CLASSIC')
#
#     test = xr.open_dataset('test_ts.nc')
#     test = test.mean(dim='time')
#     oope = test['OOPE'].values
#     length = test['length'].values
#     comm = test['community'].values.astype(np.int)
#
#     commnames = misc.extract_community_names(test)
#
#     length2d, comm2d = np.meshgrid(length, comm)
#     iok = np.nonzero(oope > 0)
#     lmin, lmax = length2d[iok].min(), length2d[iok].max()
#     rmin, rmax = oope[iok].min(), oope[iok].max()
#
#     file_pattern = 'data/PISCES*nc'
#     time, dataPHY2 = extract_ltl_data(file_pattern, 'PHY2', meshfile, domain)
#     time, dataZOO = extract_ltl_data(file_pattern, 'ZOO', meshfile, domain)
#     time, dataZOO2 = extract_ltl_data(file_pattern, 'ZOO2', meshfile, domain)
#     time, dataGOC = extract_ltl_data(file_pattern, 'GOC', meshfile, domain)
#
#     [dataPHY2, dataZOO, dataZOO2, dataGOC] = map(np.mean, [dataPHY2, dataZOO, dataZOO2, dataGOC])
#
#     import constants
#     mul = 1e-3 * constants.C_E_convert
#
#     L = [10e-6, 100e-6]
#     lVec2dPHY2, rhoLPHY2 = compute_spectra_ltl(mul*dataPHY2, L, out='length')
#
#     L = [20.e-6, 200.e-6]
#     lVec2dZOO, rhoLZOO = compute_spectra_ltl(mul*dataZOO, L, out='length')
#
#     L = [200.e-6, 2000.e-6]
#     lVec2dZOO2, rhoLZOO2 = compute_spectra_ltl(mul*dataZOO2, L, out='length')
#
#     L = [100e-6, 50000.e-6]
#     lVec2dGOC, rhoLGOC = compute_spectra_ltl(mul*dataGOC, L, out='length')
#
#
#     import pylab as plt
#     cmap = plt.cm.Spectral
#     plt.figure()
#     ax = plt.gca()
#     for icom in comm:
#         color = cmap(float(icom) / len(comm))
#         plt.scatter(length2d[icom], oope[icom], c=color, label=commnames[icom])
#
#
#     plt.scatter(lVec2dPHY2, rhoLPHY2, label='PHY2')
#     plt.scatter(lVec2dZOO, rhoLZOO, label='ZOO')
#     plt.scatter(lVec2dZOO2, rhoLZOO2, label='ZOO2')
#     plt.scatter(lVec2dGOC, rhoLGOC, label='GOC')
#
#     xmin = np.min(map(np.min, [lVec2dPHY2, lVec2dZOO, lVec2dZOO2, lVec2dGOC, length2d]))
#     xmax = np.max(map(np.max, [lVec2dPHY2, lVec2dZOO, lVec2dZOO2, lVec2dGOC, length2d]))
#
#     ymin = np.min(map(np.min, [rhoLPHY2, rhoLZOO, rhoLZOO2, rhoLGOC, oope]))
#     ymax = np.max(map(np.max, [rhoLPHY2, rhoLZOO, rhoLZOO2, rhoLGOC, oope]))
#
#     ax.set_yscale("log")
#     ax.set_xscale("log")
#     plt.xlabel('Size (m)')
#     plt.xlim(xmin, xmax)
#     plt.ylim(ymin, ymax)
#     plt.ylabel(r'$\displaystyle \rho$')
#     plt.legend()
#     plt.title(domain)
#
#     plt.savefig('size_spectra_%s.png' %domain, bbox_inches='tight')



'''
Module that contains pre-defined domains and functions
related to domains
'''

import numpy as np
try:
    from mpl_toolkits.basemap import Basemap
    import pylab as plt
except ImportError:
    pass
    

# Defintions of some domains (list to be completed)
BENGUELA = {'lon': [10, 20, 20, 10, 10], 'lat': [-36, -36, -15, -15, -36]}
OMZ_PERU = {'lon': [-90, -70, -70, -90, -90], 'lat': [-20, -20, 5, 5, -20]}
ARCTIC = {'lon': [-180, 180, 180, -180, -180], 'lat': [66, 66, 90, 90, 66]}
ANTARCTIC = {'lon': [-180, 180, 180, -180, -180], 'lat': [-55, -55, -90, -90, -55]}
ETP = {'lon': [-160, -70., -70, -160, -160], 'lat': [-20, -20, 25, 25, -20]}


DOMAINS = {'BENGUELA': BENGUELA,
           'OMZ-PERU': OMZ_PERU,
           'ARCTIC': ARCTIC,
           'ANTARCTIC': ANTARCTIC,
           'ETP': ETP}


def inpolygon(xin_2d, yin_2d, x_pol, y_pol):

    """
    Determines whether points of a 2D-grid are within a polygon.

    Equivalent to the inpolygon function of Matlab.

    .. note:: If the input polygon is not closed, it is automatically closed

    :param numpy.array xin_2d: 2-D array with the x-coords of the domain
    :param numpy.array yin_2d: 2-D array with the y-coords of the domain
    :param numpy.array x_pol: 1-D array with the x-coords of the polygon
    :param numpy.array y_pol: 1-D array with the y-coords of the polygon

    :return: A 2D array (same shape as xin_2d and yin_2d)
     with 1 when the point is within the polygon, else 0.
    :rtype: numpy.array

    """

    from matplotlib import path

    x_pol = np.array(x_pol)
    y_pol = np.array(y_pol)

    if xin_2d.ndim != 2:
        raise ValueError("The xin_2d argument must be 2D. %d dimensions" %xin_2d.ndim)

    if yin_2d.ndim != 2:
        raise ValueError("The yin_2d argument must be 2D. %d dimensions" %yin_2d.ndim)

    if x_pol.ndim != 1:
        raise ValueError("The x_pol argument must be 1D. %d dimensions" %x_pol.ndim)

    if y_pol.ndim != 1:
        raise ValueError("The y_pol argument must be 1D. %d dimensions" %y_pol.ndim)

    x_pol = np.array(x_pol)
    y_pol = np.array(y_pol)

    # If the polynom is not closed, we close it
    if (x_pol[0] != x_pol[-1]) | (y_pol[0] != y_pol[-1]):
        x_pol = np.append(x_pol, x_pol[0])
        y_pol = np.append(y_pol, y_pol[0])

    nx, ny = xin_2d.shape

    # creation the input of the path.Path command:
    # [(x1, y1), (x2, y2), (x3, y3)]
    path_input = [(xtemp, ytemp) for xtemp, ytemp in zip(x_pol, y_pol)]

    # initialisation of the path object
    temppath = path.Path(path_input)

    # creation of the list of all the points within the domain
    # it must have a N x 2 shape
    list_of_points = np.array([np.ravel(xin_2d), np.ravel(yin_2d)]).T

    # Calculation of the mask (True if within the polygon)
    mask = temppath.contains_points(list_of_points)

    # reconverting the mask into a nx by ny array
    mask = np.reshape(mask, (nx, ny))

    return mask


def plot_domains(filename='apecosm_domains.pdf', ncol=2, leg_font_size=8):

    '''
    Plots the pre-defined domains on a global map.

    :param str filename: Name of the output image file
    :param int ncol: Number of columns in the legend
    :param float leg_font_size: Legend font size

    '''

    cmap = getattr(plt.cm, plt.rcParams['image.cmap'])
    plt.figure()
    m = Basemap()
    cpt = 0.0
    for k, v in DOMAINS.items():
        col = cmap(cpt / len(DOMAINS.items()))
        m.plot(v['lon'], v['lat'], label=k, linewidth=1, color=col)
        cpt += 1.0

    m.drawcoastlines(linewidth=0.5)
    plt.legend(fontsize=leg_font_size, ncol=ncol)
    plt.savefig(filename, bbox_inches='tight')




# if __name__ == '__main__':
#
#     from envtoolkit import map as nbmap
#     import pylab as plt
#
#     data = xr.open_dataset("data/dyna_grid_T.nc")
#     sst = data['sosstsst'].values
#     lon = data['nav_lon'].values
#     lat = data['nav_lat'].values
#     sst = np.ma.masked_where(sst==0, sst)
#
#     sst = sst[0]
#
#     temp = arctic
#     temp = antarctic
#     temp = benguela
#     temp = omz_peru
#     temp = etp
#
#     mask = inpolygon(lon, lat, temp['lon'], temp['lat'])
#     print mask.shape
#
#     plt.figure()
#     plt.subplot(211)
#     cs = plt.imshow(sst, origin='lower', interpolation='none')
#     plt.colorbar(cs)
#     plt.subplot(212)
#     sst = np.ma.masked_where(mask==0, sst)
#     cs = plt.imshow(sst, origin='lower', interpolation='none')
#     plt.colorbar(cs)
#     plt.savefig('toto')
#
#     plt.figure()
#     lon = [-180, 180]
#     lat = [-90, 90]
#     m = nbmap.make_bmap(lon, lat)
#     m.plot(benguela['lon'], benguela['lat'], label='Benguela')
#     m.plot(etp['lon'], etp['lat'], label='etp')
#     m.plot(omz_peru['lon'], omz_peru['lat'], label='OMZ')
#     m.plot(arctic['lon'], arctic['lat'], label='Arctic')
#     m.plot(antarctic['lon'], antarctic['lat'], label='Antarctic')
#     m.drawcoastlines()
#     plt.legend()
#     plt.savefig('domains.png')
#
#     plot_domains()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Setup for packaging Apecosm package"
"""

__docformat__ = "restructuredtext en"

import os
from setuptools import setup, find_packages

VERSION_FILE = 'VERSION'
with open(VERSION_FILE) as fv:
    version = fv.read().strip()

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="apecosm",
    version=version,
    author="Apecosm team",
    author_email="nicolas.barrier@ird.fr",
    maintainer='Nicolas Barrier',
    maintainer_email='nicolas.barrier@ird.fr',
    description="Python package for the analysis of Apecosm outputs",
    long_description_content_type="text/markdown",
    keywords="ocean; grid model; ocean ecosystem; biology; deb",
    include_package_data=True,
    url="https://github.com/apecosm/apecosm-python",
    packages=find_packages(),
    install_requires=['xarray>=0.1',
                      'numpy>=1.9',
                      'netCDF4>=1.1', 
                      'matplotlib>=1.4',
                      'basemap>=1.0',
                      'pandas'
                     ],

    long_description = long_description,

    classifiers = [
        #"Development Status :: 5 - Production/Stable",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Mathematics",
        "Topic :: Scientific/Engineering :: Visualization",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Unix",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],

    # ++ test_suite =
    # ++ download_url
    platforms=['linux', 'mac osx'],
)
